package net.blumia.pcmdroid.ui

import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.runtime.livedata.observeAsState
import androidx.navigation.NavHostController
import androidx.navigation.compose.NavHost
import androidx.navigation.compose.composable
import androidx.navigation.compose.rememberNavController
import net.blumia.pcmdroid.MainViewModel
import net.blumia.pcmdroid.model.Folder
import net.blumia.pcmdroid.model.Server
import net.blumia.pcmdroid.model.Song
import net.blumia.pcmdroid.ui.screen.addserver.AddServerScreen
import net.blumia.pcmdroid.ui.screen.main.MainPlayer
import net.blumia.pcmdroid.ui.screen.settings.SettingsScreen

object MainDestinations {
    const val MAIN_ROUTE = "main"
    const val ADD_SERVER_ROUTE = "add_server"
    const val SETTINGS_ROUTE = "settings"
}

@Composable
fun NavGraph(
    viewModel: MainViewModel,
    navController: NavHostController = rememberNavController(),
    startDestination: String = MainDestinations.MAIN_ROUTE,
) {
    NavHost(
        navController = navController,
        startDestination = startDestination
    ) {
        composable(MainDestinations.MAIN_ROUTE) {

            val curServer: Server? by viewModel.currentServer.observeAsState(initial = null)
            val servers: List<Server> by viewModel.servers.observeAsState(initial = listOf())
            val drawerFolders: List<Folder> by viewModel.drawerFolders.observeAsState(initial = listOf())
            val curFolder: Folder? by viewModel.currentFolder.observeAsState(initial = null)
            val folders: List<Folder> by viewModel.folders.observeAsState(listOf())
            val songs: List<Song> by viewModel.songs.observeAsState(listOf())

            MainPlayer(
                currentServer = curServer,
                servers = servers,
                drawerFolders = drawerFolders,
                currentFolder = curFolder,
                folders = folders,
                songs = songs,
                addServerActionTriggered = {
                    navController.navigate(MainDestinations.ADD_SERVER_ROUTE)
                },
                onServerItemIconClicked = { server ->
                    viewModel.fetchServer(server)
                },
                onFolderItemClicked = { folder ->
                    viewModel.fetchFolder(folder)
                },
                settingsActionTriggered = {
                    navController.navigate(MainDestinations.SETTINGS_ROUTE)
                }
            )
        }

        composable(MainDestinations.ADD_SERVER_ROUTE) {
            AddServerScreen(
                onCloseActionTriggered = {
                    navController.navigateUp()
                }
            )
        }

        composable(MainDestinations.SETTINGS_ROUTE) {
            SettingsScreen(
                onBackActionTriggered = {
                    navController.navigateUp()
                }
            )
        }
    }
}